@@ -1,8 +1,18 @@
 """Provide a model for the Z-Wave JS controller."""
 from dataclasses import dataclass
-from typing import TYPE_CHECKING, Dict, List, Literal, Optional, TypedDict, Union, cast
+from typing import (
+    TYPE_CHECKING,
+    Any,
+    Dict,
+    List,
+    Literal,
+    Optional,
+    TypedDict,
+    Union,
+    cast,
+)
 
-from ..const import InclusionStrategy, SecurityClass
+from ..const import InclusionStrategy, Protocols, QRCodeVersion, SecurityClass
 from ..event import Event, EventBase
 from .association import Association, AssociationGroup
 from .node import Node
@@ -15,7 +25,7 @@ class InclusionGrantDataType(TypedDict):
     """Representation of an inclusion grant data dict type."""
 
     # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/controller/Inclusion.ts#L48-L56
-    securityClasses: List[SecurityClass]
+    securityClasses: List[int]
     clientSideAuth: bool
 
 
@@ -29,7 +39,7 @@ class InclusionGrant:
     def to_dict(self) -> InclusionGrantDataType:
         """Return InclusionGrantDataType dict from self."""
         return {
-            "securityClasses": self.security_classes,
+            "securityClasses": [sec_cls.value for sec_cls in self.security_classes],
             "clientSideAuth": self.client_side_auth,
         }
 
@@ -37,8 +47,127 @@ class InclusionGrant:
     def from_dict(cls, data: InclusionGrantDataType) -> "InclusionGrant":
         """Return InclusionGrant from InclusionGrantDataType dict."""
         return cls(
-            [SecurityClass(sec_cls) for sec_cls in data["securityClasses"]],
-            data["clientSideAuth"],
+            security_classes=[
+                SecurityClass(sec_cls) for sec_cls in data["securityClasses"]
+            ],
+            client_side_auth=data["clientSideAuth"],
+        )
+
+
+@dataclass
+class ProvisioningEntry:
+    """Representation of a provisioning entry."""
+
+    dsk: str
+    security_classes: List[SecurityClass]
+    additional_properties: Optional[Dict[str, Any]] = None
+
+    def to_dict(self) -> Dict[str, Any]:
+        """Return PlannedProvisioning data dict from self."""
+        return {
+            "dsk": self.dsk,
+            "securityClasses": [sec_cls.value for sec_cls in self.security_classes],
+            **(self.additional_properties or {}),
+        }
+
+    @classmethod
+    def from_dict(cls, data: Dict[str, Any]) -> "ProvisioningEntry":
+        """Return ProvisioningEntry from data dict."""
+        return cls(
+            dsk=data["dsk"],
+            security_classes=[
+                SecurityClass(sec_cls) for sec_cls in data["securityClasses"]
+            ],
+            additional_properties={
+                k: v for k, v in data.items() if k not in {"dsk", "securityClasses"}
+            },
+        )
+
+
+class QRProvisioningInformationDataType(TypedDict, total=False):
+    """Representation of provisioning information data dict type retrieved from a QR code."""
+
+    version: int  # required
+    securityClasses: List[int]  # required
+    dsk: str  # required
+    genericDeviceClass: int  # required
+    specificDeviceClass: int  # required
+    installerIconType: int  # required
+    manufacturerId: int  # required
+    productType: int  # required
+    productId: int  # required
+    applicationVersion: str  # required
+    maxInclusionRequestInterval: int
+    uuid: str
+    supportedProtocols: List[int]
+
+
+@dataclass
+class QRProvisioningInformation:
+    """Representation of provisioning information retrieved from a QR code."""
+
+    version: QRCodeVersion
+    security_classes: List[SecurityClass]
+    dsk: str
+    generic_device_class: int
+    specific_device_class: int
+    installer_icon_type: int
+    manufacturer_id: int
+    product_type: int
+    product_id: int
+    application_version: str
+    max_inclusion_request_interval: Optional[int]
+    uuid: Optional[str]
+    supported_protocols: Optional[List[Protocols]]
+
+    def to_dict(self) -> QRProvisioningInformationDataType:
+        """Return QRProvisioningInformationDataType dict from self."""
+        data = QRProvisioningInformationDataType(
+            version=self.version.value,
+            securityClasses=[sec_cls.value for sec_cls in self.security_classes],
+            dsk=self.dsk,
+            genericDeviceClass=self.generic_device_class,
+            specificDeviceClass=self.specific_device_class,
+            installerIconType=self.installer_icon_type,
+            manufacturerId=self.manufacturer_id,
+            productType=self.product_type,
+            productId=self.product_id,
+            applicationVersion=self.application_version,
+        )
+        if self.max_inclusion_request_interval is not None:
+            data["maxInclusionRequestInterval"] = self.max_inclusion_request_interval
+        if self.uuid is not None:
+            data["uuid"] = self.uuid
+        if self.supported_protocols is not None:
+            data["supportedProtocols"] = [
+                protocol.value for protocol in self.supported_protocols
+            ]
+        return data
+
+    @classmethod
+    def from_dict(
+        cls, data: QRProvisioningInformationDataType
+    ) -> "QRProvisioningInformation":
+        """Return QRProvisioningInformation from QRProvisioningInformationDataType dict."""
+        return cls(
+            version=QRCodeVersion(data["version"]),
+            security_classes=[
+                SecurityClass(sec_cls) for sec_cls in data["securityClasses"]
+            ],
+            dsk=data["dsk"],
+            generic_device_class=data["genericDeviceClass"],
+            specific_device_class=data["specificDeviceClass"],
+            installer_icon_type=data["installerIconType"],
+            manufacturer_id=data["manufacturerId"],
+            product_type=data["productType"],
+            product_id=data["productId"],
+            application_version=data["applicationVersion"],
+            max_inclusion_request_interval=data.get("maxInclusionRequestInterval"),
+            uuid=data.get("uuid"),
+            supported_protocols=[
+                Protocols(supported_protocol)
+                for supported_protocol in data.get("supportedProtocols", [])
+            ],
         )
 
 
@@ -279,38 +408,105 @@ class Controller(EventBase):
 
     async def async_begin_inclusion(
         self,
-        inclusion_strategy: Union[
-            Literal[InclusionStrategy.SECURITY_S0],
-            Literal[InclusionStrategy.SECURITY_S2],
-            Literal[InclusionStrategy.INSECURE],
+        inclusion_strategy: Literal[
+            InclusionStrategy.DEFAULT,
+            InclusionStrategy.SECURITY_S0,
+            InclusionStrategy.SECURITY_S2,
+            InclusionStrategy.INSECURE,
         ],
+        force_security: Optional[bool] = None,
+        provisioning: Optional[
+            Union[str, ProvisioningEntry, QRProvisioningInformation]
+        ] = None,
     ) -> bool:
         """Send beginInclusion command to Controller."""
+        # Most functionality was introduced in Schema 8
+        require_schema = 8
+        options: Dict[str, Any] = {"strategy": inclusion_strategy}
+        # forceSecurity can only be used with the default inclusion strategy
+        if force_security is not None:
+            if inclusion_strategy != InclusionStrategy.DEFAULT:
+                raise ValueError(
+                    "`forceSecurity` option is only supported with inclusion_strategy=DEFAULT"
+                )
+            options["forceSecurity"] = force_security
+
+        # provisioning can only be used with the S2 inclusion strategy and may need
+        # additional processing
+        if provisioning is not None:
+            if inclusion_strategy != InclusionStrategy.SECURITY_S2:
+                raise ValueError(
+                    "`provisioning` option is only supported with inclusion_strategy=SECURITY_S2"
+                )
+            # Provisioning option was introduced in Schema 11
+            require_schema = 11
+            # String is assumed to be the QR code string so we can pass as is
+            if isinstance(provisioning, str):
+                options["provisioning"] = provisioning
+            # Otherwise we assume the data is ProvisioningEntry or
+            # QRProvisioningInformation
+            else:
+                options["provisioning"] = provisioning.to_dict()
+
         data = await self.client.async_send_command(
             {
                 "command": "controller.begin_inclusion",
-                "options": {"strategy": inclusion_strategy},
+                "options": options,
             },
-            require_schema=8,
+            require_schema=require_schema,
         )
         return cast(bool, data["success"])
 
-    async def async_begin_inclusion_default(
+    async def async_provision_smart_start_node(
         self,
-        force_security: Optional[bool] = None,
-    ) -> bool:
-        """Send beginInclusion command to Controller using Default inclusion method."""
+        provisioning_info: Union[ProvisioningEntry, QRProvisioningInformation, str],
+    ) -> None:
+        """Send provisionSmartStartNode command to Controller."""
+        await self.client.async_send_command(
+            {
+                "command": "controller.provision_smart_start_node",
+                "entry": provisioning_info
+                if isinstance(provisioning_info, str)
+                else provisioning_info.to_dict(),
+            },
+            require_schema=11,
+        )
+
+    async def async_unprovision_smart_start_node(
+        self,
+        dsk_or_node_id: Union[str, int],
+    ) -> None:
+        """Send unprovisionSmartStartNode command to Controller."""
+        await self.client.async_send_command(
+            {
+                "command": "controller.unprovision_smart_start_node",
+                "dskOrNodeId": dsk_or_node_id,
+            },
+            require_schema=11,
+        )
+
+    async def async_get_provisioning_entry(
+        self, dsk: str
+    ) -> Optional[ProvisioningEntry]:
+        """Send getProvisioningEntry command to Controller."""
         data = await self.client.async_send_command(
             {
-                "command": "controller.begin_inclusion",
-                "options": {
-                    "strategy": InclusionStrategy.DEFAULT,
-                    "forceSecurity": force_security,
-                },
+                "command": "controller.get_provisioning_entry",
+                "dsk": dsk,
             },
-            require_schema=8,
+            require_schema=11,
         )
-        return cast(bool, data["success"])
+        return ProvisioningEntry.from_dict(data["entry"])
+
+    async def async_get_provisioning_entries(self) -> List[ProvisioningEntry]:
+        """Send getProvisioningEntries command to Controller."""
+        data = await self.client.async_send_command(
+            {
+                "command": "controller.get_provisioning_entries",
+            },
+            require_schema=11,
+        )
+        return [ProvisioningEntry.from_dict(entry) for entry in data.get("entries", [])]
 
     async def async_stop_inclusion(self) -> bool:
         """Send stopInclusion command to Controller."""
@@ -319,11 +515,12 @@ class Controller(EventBase):
         )
         return cast(bool, data["success"])
 
-    async def async_begin_exclusion(self) -> bool:
+    async def async_begin_exclusion(self, unprovision: Optional[bool] = None) -> bool:
         """Send beginExclusion command to Controller."""
-        data = await self.client.async_send_command(
-            {"command": "controller.begin_exclusion"}
-        )
+        payload: Dict[str, Union[str, bool]] = {"command": "controller.begin_exclusion"}
+        if unprovision is not None:
+            payload["unprovision"] = unprovision
+        data = await self.client.async_send_command(payload)
         return cast(bool, data["success"])
 
     async def async_stop_exclusion(self) -> bool:
@@ -342,20 +539,52 @@ class Controller(EventBase):
     async def async_replace_failed_node(
         self,
         node_id: int,
-        inclusion_strategy: Union[
-            Literal[InclusionStrategy.SECURITY_S0],
-            Literal[InclusionStrategy.SECURITY_S2],
-            Literal[InclusionStrategy.INSECURE],
+        inclusion_strategy: Literal[
+            InclusionStrategy.DEFAULT,
+            InclusionStrategy.SECURITY_S0,
+            InclusionStrategy.SECURITY_S2,
+            InclusionStrategy.INSECURE,
         ],
+        force_security: Optional[bool] = None,
+        provisioning: Optional[
+            Union[str, ProvisioningEntry, QRProvisioningInformation]
+        ] = None,
     ) -> bool:
         """Send replaceFailedNode command to Controller."""
+        # Most functionality was introduced in Schema 8
+        require_schema = 8
+        options: Dict[str, Any] = {"strategy": inclusion_strategy}
+        # forceSecurity can only be used with the default inclusion strategy
+        if force_security is not None:
+            if inclusion_strategy != InclusionStrategy.DEFAULT:
+                raise ValueError(
+                    "`forceSecurity` option is only supported with inclusion_strategy=DEFAULT"
+                )
+            options["forceSecurity"] = force_security
+
+        # provisioning can only be used with the S2 inclusion strategy and may need
+        # additional processing
+        if provisioning is not None:
+            if inclusion_strategy != InclusionStrategy.SECURITY_S2:
+                raise ValueError(
+                    "`provisioning` option is only supported with inclusion_strategy=SECURITY_S2"
+                )
+            # Provisioning option was introduced in Schema 11
+            require_schema = 11
+            # String is assumed to be the QR code string so we can pass as is
+            if isinstance(provisioning, str):
+                options["provisioning"] = provisioning
+            # Otherwise we assume the data is ProvisioningEntry or
+            # QRProvisioningInformation
+            else:
+                options["provisioning"] = provisioning.to_dict()
         data = await self.client.async_send_command(
             {
                 "command": "controller.replace_failed_node",
                 "nodeId": node_id,
-                "options": {"strategy": inclusion_strategy},
+                "options": options,
             },
-            require_schema=8,
+            require_schema=require_schema,
         )
         return cast(bool, data["success"])
 
